package org.vraptor.http;

import java.util.Enumeration;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.vraptor.LogicRequest;
import org.vraptor.core.VRaptorController;
import org.vraptor.factory.FactoryException;

/**
 * A simple request wrapper for dealing with variable lookup.
 * 
 * @author Guilherme Silveira
 */
public class VRaptorServletRequest extends HttpServletRequestWrapper {

	private LogicRequest logicRequest;

	private final VRaptorController controller;

	private final RequestParameters parameters;

	/**
	 * Constructor for this wrapper
	 * 
	 * @param req
	 *            request
	 * @param controller
	 *            the controller
	 */
	@SuppressWarnings("unchecked")
	public VRaptorServletRequest(HttpServletRequest req, VRaptorController controller) {
		super(req);
		this.parameters = new RequestParameters(req.getParameterMap());
		this.controller = controller;
	}

	@Override
	public Object getAttribute(String key) {
		if (super.getAttribute(key) != null
				|| (!controller.getWebApplication().getFactoryManager().canInstantiate(key))) {
			return super.getAttribute(key);
		}
		return instantiate(key);
	}

	/**
	 * Instantiates the desired key
	 * 
	 * @param key
	 *            desired key
	 * @return the object
	 */
	private Object instantiate(String key) {
		try {
			return controller.getWebApplication().getFactoryManager().instantiate(key, this.logicRequest,
					this.controller.getIntrospector());
		} catch (FactoryException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Changes the current logicRequest
	 * 
	 * @param logicRequest
	 *            logicRequest
	 */
	public void setCurrentRequest(LogicRequest context) {
		this.logicRequest = context;
	}

	public void overwriteParameters(String name, String value) {
		this.parameters.put(name, value);
	}

	@Override
	public String getParameter(String name) {
		return parameters.getParameter(name);
	}

	@Override
	public Map getParameterMap() {
		return parameters.getParameterMap();
	}

	@Override
	public Enumeration getParameterNames() {
		return parameters.getParameterNames();
	}

	@Override
	public String[] getParameterValues(String name) {
		return parameters.getParameterValues(name);
	}

}
