package org.vraptor.plugin.interceptor;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.annotations.InterceptedBy;
import org.vraptor.interceptor.InterceptorType;
import org.vraptor.introspector.Introspector;
import org.vraptor.view.ViewException;

/**
 * A method interceptor capability.
 *
 * @author Guilherme Silveira
 * @since 2.3.2
 */
public class MethodInterceptorInterceptor implements Interceptor {

	public void intercept(LogicFlow flow) throws LogicException, ViewException {
		Method method = flow.getLogicRequest().getLogicMethod().getMetadata().getMethod();
		if (method.isAnnotationPresent(InterceptedBy.class)) {
			// injects a few more interceptors here before finishing
			InterceptedBy annotation = method.getAnnotation(InterceptedBy.class);
			Introspector introspector = (Introspector) flow.getLogicRequest().getApplicationContext().getAttribute(
					Introspector.INTROSPECTOR_KEY);
			List<InterceptorType> interceptors = new ArrayList<InterceptorType>();
			for (Class interceptorType : annotation.value()) {
				interceptors.add(InterceptorType.getType(interceptorType));
			}
			new ConcatenateInterceptorsLogicFlow(interceptors, flow, introspector).execute();
		} else {
			flow.execute();
		}
	}

}
