package org.vraptor.plugin.spring;

import org.apache.log4j.Logger;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.vraptor.introspector.Introspector;
import org.vraptor.plugin.VRaptorPlugin;
import org.vraptor.webapp.WebApplication;

/**
 * Spring plugin which reads the application bean configuration and allows it to
 * be used for VRaptor's dependencies injection.
 * 
 * @author Guilherme Silveira
 * @author Fabio Patricio
 * @since 2.2.4
 */
public class SpringPlugin implements VRaptorPlugin {

	private static final String SPRING_CONTEXT_KEY = "springContext";

	private static final String APPLICATION_CONTEXT_XML = "applicationContext.xml";

	private static final Logger LOG = Logger.getLogger(SpringPlugin.class);

	public void init(WebApplication application) {
		long startTime = System.currentTimeMillis();
		LOG.info("Loading Spring root WebApplicationContext");

		ApplicationContext context = new ClassPathXmlApplicationContext(APPLICATION_CONTEXT_XML);
		application.getApplicationContext().setAttribute(SPRING_CONTEXT_KEY, context);
		application.getApplicationContext().setAttribute(ApplicationContext.class.getName(), context);

		if (LOG.isDebugEnabled()) {
			LOG.debug("Published root WebApplicationContext [" + context + "] as ServletContext attribute");
		}
		if (LOG.isInfoEnabled()) {
			LOG.info("Using context class [" + context.getClass().getName() + "] for root ApplicationContext");
			long elapsedTime = System.currentTimeMillis() - startTime;
			LOG.info("Root ApplicationContext: initialization completed in " + elapsedTime + " ms");
		}

		Introspector introspector = application.getIntrospector();
		SpringProvider provider = new SpringProvider(introspector.getBeanProvider(), context);
		LOG.debug("Registering new spring provider with the introspector");
		introspector.setBeanProvider(provider);
	}

}
