package org.vraptor.scope;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.LogicRequest;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicMethod;
import org.vraptor.core.WebRequest;
import org.vraptor.url.RequestInfo;

/**
 * Basic vraptor logic request.
 * 
 * @author Guilherme Silveira
 */
public class DefaultLogicRequest implements LogicRequest {

	private static final Logger LOG = Logger.getLogger(DefaultLogicRequest.class);

	private ApplicationContext applicationContext;

	private SessionContext sessionContext;

	private RequestContext requestContext;

	private final LogicContext logicContext;

	private String result;

	private final RequestInfo info;

	private Object component;

	private final ComponentType componentType;

	private final LogicMethod logicMethod;

	private final WebRequest request;

	/**
	 * Creates the logic context
	 * 
	 * @param viewType
	 * 
	 * @param request
	 *            request
	 * @param response
	 *            response
	 * @param application
	 *            application scope
	 * @param method
	 * @param componentType
	 * 
	 */
	public DefaultLogicRequest(RequestInfo info, WebRequest request, ComponentType componentType, LogicMethod method) {
		this.request = request;
		this.info = info;
		this.applicationContext = new SimpleApplicationContext(request.getContext());
		this.componentType = componentType;
		this.logicMethod = method;
		this.logicContext = new DefaultLogicContext(request.getRequest(), request.getResponse());
	}

	public ApplicationContext getApplicationContext() {
		return this.applicationContext;
	}

	public SessionContext getSessionContext() {
		if (this.sessionContext == null) {
			this.sessionContext = new SessionContext(this.request.getRequest().getSession());
		}
		return this.sessionContext;
	}

	public RequestContext getRequestContext() {
		if (this.requestContext == null) {
			this.requestContext = new DefaultRequestContext(request.getRequest());
		}
		return this.requestContext;
	}

	public HttpServletRequest getRequest() {
		return request.getRequest();
	}

	public HttpServletResponse getResponse() {
		return request.getResponse();
	}

	public LogicContext getLogicContext() {
		return this.logicContext;
	}

	public Object findAttribute(String key) {
		if (getLogicContext().hasAttribute(key)) {
			return logicContext.getAttribute(key);
		}
		if (getRequestContext().hasAttribute(key)) {
			return requestContext.getAttribute(key);
		}
		if (getSessionContext().hasAttribute(key)) {
			return sessionContext.getAttribute(key);
		}
		if (getApplicationContext().hasAttribute(key)) {
			return applicationContext.getAttribute(key);
		}
		if (LOG.isDebugEnabled()) {
			LOG.debug("key not found in any contexts: " + key);
		}
		return null;
	}

	public void confirmResult(String string) {
		if (this.result != null) {
			throw new IllegalStateException("the result for the logic was already set");
		}
		this.result = string;
	}

	public String getResult() {
		return this.result;
	}

	public RequestInfo getRequestInfo() {
		return info;
	}

	public void setComponent(Object component) {
		if (this.component != null) {
			throw new IllegalStateException("component already set");
		}
		this.component = component;
	}

	public Object getComponent() {
		return component;
	}

	public ComponentType getComponentType() {
		return componentType;
	}

	public LogicMethod getLogicMethod() {
		return logicMethod;
	}

	public ServletContext getServletContext() {
		return request.getContext();
	}

}
